@@ -10,11 +10,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -31,10 +27,6 @@
  * SUCH DAMAGE.
  */
 
-#if defined(LIBC_SCCS) && !defined(lint)
-static char rcsid[] = "$OpenBSD: getcwd.c,v 1.5 1998/08/14 21:39:26 deraadt Exp $";
-#endif /* LIBC_SCCS and not lint */
-
 #include "config.h"
 
 #include <sys/param.h>
@@ -76,10 +68,18 @@ static char rcsid[] = "$OpenBSD: getcwd.c,v 1.5 1998/08/14 21:39:26 deraadt Exp
 
 #include "compat.h"
 
+#ifndef dirfd
+#  define dirfd(dirp)	((dirp)->dd_fd)
+#endif
+
 #define	ISDOT(dp) \
 	(dp->d_name[0] == '.' && (dp->d_name[1] == '\0' || \
 	    (dp->d_name[1] == '.' && dp->d_name[2] == '\0')))
 
+#ifndef lint
+static const char rcsid[] = "$Sudo: getcwd.c,v 1.21 1999/09/04 07:09:03 millert Exp $";
+#endif /* lint */
+
 char *
 getcwd(pt, size)
 	char *pt;
