@@ -13,11 +13,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -34,13 +30,6 @@
  * SUCH DAMAGE.
  */
 
-#if defined(LIBC_SCCS) && !defined(lint)
-static char sccsid[] = "@(#)lsearch.c	8.1 (Berkeley) 6/4/93";
-#endif /* LIBC_SCCS and not lint */
-#ifndef lint
-static char rcsid[] = "$Id: lsearch.c,v 1.12 1998/04/06 03:25:36 millert Exp $";
-#endif /* lint */
-
 #include "config.h"
 
 #include <stdio.h>
@@ -52,6 +41,12 @@ static char rcsid[] = "$Id: lsearch.c,v 1.12 1998/04/06 03:25:36 millert Exp $";
 #include "compat.h"
 #include "emul/search.h"
 
+#if defined(LIBC_SCCS) && !defined(lint)
+static const char sccsid[] = "@(#)lsearch.c	8.1 (Berkeley) 6/4/93";
+#endif /* LIBC_SCCS and not lint */
+#ifndef lint
+static const char rcsid[] = "$Sudo: lsearch.c,v 1.16 1999/09/04 07:09:03 millert Exp $";
+#endif /* lint */
 
 typedef int (*cmp_fn_t) __P((const VOID *, const VOID *));
 static VOID *linear_base __P((const VOID *, const VOID *, size_t *, size_t,
