@@ -1,98 +1,128 @@
 /*
- *  CU sudo version 1.5.9
- *  Copyright (c) 1996, 1998, 1999 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 1996, 1998, 1999 Todd C. Miller <Todd.Miller@courtesan.com>
+ * All rights reserved.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 1, or (at your option)
- *  any later version.
+ * This code is derived from software contributed by Chris Jepeway
+ * <jepeway@cs.utk.edu>.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- *  Please send bugs, changes, problems to sudo-bugs@courtesan.com
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- *******************************************************************
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
  *
- *  testsudoers.c -- frontend for parser testing and development.
+ * 4. Products derived from this software may not be called "Sudo" nor
+ *    may "Sudo" appear in their names without specific prior written
+ *    permission from the author.
  *
- *  Chris Jepeway <jepeway@cs.utk.edu>
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
+ * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "config.h"
 
 #include <stdio.h>
 #ifdef STDC_HEADERS
-#  include <stdlib.h>
+# include <stdlib.h>
 #endif /* STDC_HEADERS */
 #ifdef HAVE_UNISTD_H
-#  include <unistd.h>
+# include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 #ifdef HAVE_STRING_H
-#  include <string.h>
+# include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
-#  include <strings.h>
+# include <strings.h>
 #endif /* HAVE_STRINGS_H */
 #if defined(HAVE_FNMATCH) && defined(HAVE_FNMATCH_H)
-#  include <fnmatch.h>
-#else
-#  ifndef HAVE_FNMATCH
-#    include "emul/fnmatch.h"
-#  endif /* HAVE_FNMATCH */
+# include <fnmatch.h>
 #endif /* HAVE_FNMATCH_H */
 #ifdef HAVE_NETGROUP_H
-#  include <netgroup.h>
+# include <netgroup.h>
 #endif /* HAVE_NETGROUP_H */
 #include <ctype.h>
 #include <pwd.h>
 #include <grp.h>
 #include <sys/param.h>
 #include <sys/types.h>
+#include <sys/socket.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 #include <sys/stat.h>
 #include <dirent.h>
 
 #include "sudo.h"
+#include "parse.h"
+#include "interfaces.h"
+
+#ifndef HAVE_FNMATCH
+# include "emul/fnmatch.h"
+#endif /* HAVE_FNMATCH */
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: testsudoers.c,v 1.51 1999/03/29 04:05:13 millert Exp $";
+static const char rcsid[] = "$Sudo: testsudoers.c,v 1.64 1999/09/08 08:06:19 millert Exp $";
 #endif /* lint */
 
 /*
  * Globals
  */
-int parse_error = FALSE;
-extern int clearaliases;
-extern struct interface *interfaces;
-extern int num_interfaces;
-
-char *cmnd = NULL;
-char *cmnd_args = NULL;
-char *runas_user = RUNAS_DEFAULT;
-char host[MAXHOSTNAMELEN];
-char *shost;
-char cwd[MAXPATHLEN];
-struct passwd *user_pw_ent;
 char **Argv, **NewArgv;
 int  Argc, NewArgc;
-uid_t uid;
-
+int parse_error = FALSE;
+int num_interfaces;
+struct interface *interfaces;
+struct sudo_user sudo_user;
+extern int clearaliases;
+extern int pedantic;
 
 /*
- * return TRUE if cmnd matches, in the sudo sense,
+ * Prototypes for external functions
+ */
+void init_parser	__P((void));
+void dumpaliases	__P((void));
+
+/*
+ * Returns TRUE if "s" has shell meta characters in it,
+ * else returns FALSE.
+ */
+int
+has_meta(s)
+    char *s;
+{
+    register char *t;
+    
+    for (t = s; *t; t++) {
+	if (*t == '\\' || *t == '?' || *t == '*' || *t == '[' || *t == ']')
+	    return(TRUE);
+    }
+    return(FALSE);
+}
+
+/*
+ * Returns TRUE if cmnd matches, in the sudo sense,
  * the pathname in path; otherwise, return FALSE
  */
-int command_matches(cmnd, user_args, path, sudoers_args)
+int
+command_matches(cmnd, cmnd_args, path, sudoers_args)
     char *cmnd;
-    char *user_args;
+    char *cmnd_args;
     char *path;
     char *sudoers_args;
 {
@@ -110,10 +140,10 @@ int command_matches(cmnd, user_args, path, sudoers_args)
 	    return(FALSE);
 	if (!sudoers_args)
 	    return(TRUE);
-	else if (!user_args && sudoers_args && !strcmp("\"\"", sudoers_args))
+	else if (!cmnd_args && sudoers_args && !strcmp("\"\"", sudoers_args))
 	    return(TRUE);
 	else if (sudoers_args)
-	    return((fnmatch(sudoers_args, user_args ? user_args : "", 0) == 0));
+	    return((fnmatch(sudoers_args, cmnd_args ? cmnd_args : "", 0) == 0));
 	else
 	    return(FALSE);
     } else {
@@ -123,10 +153,10 @@ int command_matches(cmnd, user_args, path, sudoers_args)
 		return(FALSE);
 	    if (!sudoers_args)
 		return(TRUE);
-	    else if (!user_args && sudoers_args && !strcmp("\"\"", sudoers_args))
+	    else if (!cmnd_args && sudoers_args && !strcmp("\"\"", sudoers_args))
 		return(TRUE);
 	    else if (sudoers_args)
-		return((fnmatch(sudoers_args, user_args ? user_args : "", 0) == 0));
+		return((fnmatch(sudoers_args, cmnd_args ? cmnd_args : "", 0) == 0));
 	    else
 		return(FALSE);
 	}
@@ -146,19 +176,22 @@ int command_matches(cmnd, user_args, path, sudoers_args)
     }
 }
 
-
-int addr_matches(n)
+int
+addr_matches(n)
     char *n;
 {
     int i;
     char *m;
     struct in_addr addr, mask;
 
-    /* If there's an explicate netmask, use it. */
+    /* If there's an explicit netmask, use it. */
     if ((m = strchr(n, '/'))) {
 	*m++ = '\0';
-	mask.s_addr = inet_addr(m);
 	addr.s_addr = inet_addr(n);
+	if (strchr(m, '.'))
+	    mask.s_addr = inet_addr(m);
+	else
+	    mask.s_addr = (1 << atoi(m)) - 1;	/* XXX - better way? */
 	*(m - 1) = '/';               
 
 	for (i = 0; i < num_interfaces; i++)
@@ -177,15 +210,15 @@ int addr_matches(n)
     return(FALSE);
 }
 
-
-int usergr_matches(group, user)
+int
+usergr_matches(group, user)
     char *group;
     char *user;
 {
     struct group *grp;
     char **cur;
 
-    /* make sure we have a valid usergroup, sudo style */
+    /* Make sure we have a valid usergroup, sudo style. */
     if (*group++ != '%')
 	return(FALSE);
 
@@ -206,8 +239,8 @@ int usergr_matches(group, user)
     return(FALSE);
 }
 
-
-int netgr_matches(netgr, host, user)
+int
+netgr_matches(netgr, host, user)
     char *netgr;
     char *host;
     char *user;
@@ -218,17 +251,17 @@ int netgr_matches(netgr, host, user)
     static char *domain = NULL;
 #endif /* HAVE_GETDOMAINNAME */
 
-    /* make sure we have a valid netgroup, sudo style */
+    /* Make sure we have a valid netgroup, sudo style. */
     if (*netgr++ != '+')
 	return(FALSE);
 
 #ifdef HAVE_GETDOMAINNAME
-    /* get the domain name (if any) */
+    /* Get the domain name (if any). */
     if (domain == (char *) -1) {
 	domain = (char *) emalloc(MAXHOSTNAMELEN);
 
 	if (getdomainname(domain, MAXHOSTNAMELEN) != 0 || *domain == '\0') {
-	    (void) free(domain);
+	    free(domain);
 	    domain = NULL;
 	}
     }
@@ -241,15 +274,15 @@ int netgr_matches(netgr, host, user)
 #endif /* HAVE_INNETGR */
 }
 
-
-void set_perms(i, j)
+void
+set_perms(i, j)
     int i, j;
 {
     return;
 }
 
-
-int main(argc, argv)
+int
+main(argc, argv)
     int argc;
     char **argv;
 {
@@ -264,57 +297,64 @@ int main(argc, argv)
     Argc = argc;
 
     if (Argc >= 6 && strcmp(Argv[1], "-u") == 0) {
-	runas_user = Argv[2];
+	user_runas = &Argv[2];
 	pw.pw_name = Argv[3];
-	strcpy(host, Argv[4]);
-	cmnd = Argv[5];
+	user_host = Argv[4];
+	user_cmnd = Argv[5];
 
 	NewArgv = &Argv[5];
 	NewArgc = Argc - 5;
     } else if (Argc >= 4) {
 	pw.pw_name = Argv[1];
-	strcpy(host, Argv[2]);
-	cmnd = Argv[3];
+	user_host = Argv[2];
+	user_cmnd = Argv[3];
 
 	NewArgv = &Argv[3];
 	NewArgc = Argc - 3;
     } else {
-	(void) fprintf(stderr, "usage: %s [-u user] <user> <host> <command> [args]\n", Argv[0]);
+	(void) fprintf(stderr,
+	    "usage: %s [-u user] <user> <host> <command> [args]\n", Argv[0]);
 	exit(1);
     }
 
-    user_pw_ent = &pw;		/* need user_pw_ent->pw_name defined */
+    sudo_user.pw = &pw;		/* user_name needs to be defined */
 
-    if ((p = strchr(host, '.'))) {
+    if ((p = strchr(user_host, '.'))) {
 	*p = '\0';
-	shost = estrdup(host);
+	user_shost = estrdup(user_host);
 	*p = '.';
     } else {
-	shost = &host[0];
+	user_shost = user_host;
     }
 
-    /* fill in cmnd_args from NewArgv */
+    /* Fill in cmnd_args from NewArgv. */
     if (NewArgc > 1) {
 	size_t size;
 	char *to, **from;
 
 	size = (size_t) NewArgv[NewArgc-1] + strlen(NewArgv[NewArgc-1]) -
 	       (size_t) NewArgv[1] + 1;
-	cmnd_args = (char *) emalloc(size);
-	for (to = cmnd_args, from = &NewArgv[1]; *from; from++) {
+	user_args = (char *) emalloc(size);
+	for (to = user_args, from = &NewArgv[1]; *from; from++) {
 	    *to++ = ' ';
 	    (void) strcpy(to, *from);
 	    to += strlen(*from);
 	}
     }
 
-    /* need to keep aliases around for dumpaliases() */
-    clearaliases = 0;
+    /* Initialize default values. */
+    init_defaults();
 
-    /* load ip addr/mask for each interface */
+    /* Warn about aliases that are used before being defined. */
+    pedantic = TRUE;
+
+    /* Need to keep aliases around for dumpaliases(). */
+    clearaliases = FALSE;
+
+    /* Load ip addr/mask for each interface. */
     load_interfaces();
 
-    /* allocate space for data structures in the parser */
+    /* Allocate space for data structures in the parser. */
     init_parser();
 
     if (yyparse() || parse_error) {
@@ -330,31 +370,14 @@ int main(argc, argv)
 	    (void) printf("cmnd_match : %d\n", cmnd_matches);
 	    (void) printf("no_passwd  : %d\n", no_passwd);
 	    (void) printf("runas_match: %d\n", runas_matches);
-	    (void) printf("runas      : %s\n", runas_user);
+	    (void) printf("runas      : %s\n", *user_runas);
 	    top--;
 	}
     }
 
-    /* dump aliases */
+    /* Dump aliases. */
     (void) printf("Matching Aliases --\n");
     dumpaliases();
 
     exit(0);
 }
-
-
-/*
- * Returns TRUE if "s" has shell meta characters in it,
- * else returns FALSE.
- */
-int has_meta(s)
-    char *s;
-{
-    register char *t;
-    
-    for (t = s; *t; t++) {
-	if (*t == '\\' || *t == '?' || *t == '*' || *t == '[' || *t == ']')
-	    return(TRUE);
-    }
-    return(FALSE);
-}
