@@ -1,29 +1,39 @@
 /*
- *  CU sudo version 1.5.9
- *  Copyright (c) 1996, 1998, 1999 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 1996, 1998, 1999 Todd C. Miller <Todd.Miller@courtesan.com>
+ * All rights reserved.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 1, or (at your option)
- *  any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- *  Please send bugs, changes, problems to sudo-bugs@courtesan.com
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
  *
- *******************************************************************
+ * 4. Products derived from this software may not be called "Sudo" nor
+ *    may "Sudo" appear in their names without specific prior written
+ *    permission from the author.
  *
- *  visudo.c -- locks the sudoers file for safe editing and check
- *  for parse errors.
- *
- *  Todd C. Miller <Todd.Miller@courtesan.com> Sat Mar 25 21:50:36 MST 1995
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
+ * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+/*
+ * Lock the sudoers file for safe editing (ala vipw) and check for parse errors.
  */
 
 #include "config.h"
@@ -46,6 +56,7 @@
 #endif /* HAVE_MALLOC_H && !STDC_HEADERS */
 #include <ctype.h>
 #include <pwd.h>
+#include <time.h>
 #include <signal.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -53,7 +64,6 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/file.h>
-#include <netinet/in.h>
 
 #include "sudo.h"
 #include "version.h"
@@ -71,7 +81,7 @@ extern int stat		__P((const char *, struct stat *));
 #endif /* POSIX_SIGNALS && !SA_RESETHAND */
 
 #ifndef lint
-static const char rcsid[] = "$Sudo: visudo.c,v 1.101 1999/04/02 21:02:57 millert Exp $";
+static const char rcsid[] = "$Sudo: visudo.c,v 1.116 1999/11/09 20:12:20 millert Exp $";
 #endif /* lint */
 
 /*
@@ -79,7 +89,6 @@ static const char rcsid[] = "$Sudo: visudo.c,v 1.101 1999/04/02 21:02:57 millert
  */
 static void usage		__P((void));
 static char whatnow		__P((void));
-static void whatnow_help	__P((void));
 static RETSIGTYPE Exit		__P((int));
 static void setup_signals	__P((void));
 int command_matches		__P((char *, char *, char *, char *));
@@ -87,54 +96,40 @@ int addr_matches		__P((char *));
 int netgr_matches		__P((char *, char *, char *));
 int usergr_matches		__P((char *, char *));
 void init_parser		__P((void));
-
+void yyrestart			__P((FILE *));
 
 /*
- * External globals
+ * External globals exported by the parser
  */
 extern FILE *yyin, *yyout;
-extern int errorlineno, sudolineno;
-
+extern int errorlineno;
+extern int pedantic;
 
 /*
  * Globals
  */
 char **Argv;
-char **NewArgv = NULL;
-int NewArgc = 0;
-char *sudoers = _PATH_SUDO_SUDOERS;
-char *stmp = _PATH_SUDO_STMP;
+char *sudoers = _PATH_SUDOERS;
+char *stmp = _PATH_SUDOERS_TMP;
+struct sudo_user sudo_user;
 int parse_error = FALSE;
-char *runas_user = RUNAS_DEFAULT;
-
-/*
- * For the parsing routines
- */
-char host[] = "";
-char *shost = "";
-char *cmnd = "";
-char *cmnd_args = NULL;
-struct passwd *user_pw_ent;
 
 
-/********************************************************************
- *
- *  main()
- *
- *  where it all begins...
- */
-
-int main(argc, argv)
+int
+main(argc, argv)
     int argc;
     char **argv;
 {
     char buf[MAXPATHLEN*2];		/* buffer used for copying files */
-    char * Editor = EDITOR;		/* editor to use (default is EDITOR */
+    char *Editor = EDITOR;		/* editor to use (default is EDITOR */
     int sudoers_fd;			/* sudoers file descriptor */
     int stmp_fd;			/* stmp file descriptor */
     int n;				/* length parameter */
+    time_t now;				/* time now */
+    struct stat stmp_sb, sudoers_sb;	/* to check for changes */
 
-    (void) setbuf(stderr, (char *)NULL);	/* unbuffered stderr */
+    /* Warn about aliases that are used before being defined. */
+    pedantic = 1;
 
     /*
      * Parse command line options
@@ -142,25 +137,24 @@ int main(argc, argv)
     Argv = argv;
 
     /*
-     * If passesd -V then print version, else print usage
-     * if any other option...
+     * Arg handling.
      */
-    if (argc == 2) {
-	if (!strcmp(Argv[1], "-V")) {
+    while (--argc) {
+	if (!strcmp(argv[argc], "-V")) {
 	    (void) printf("visudo version %s\n", version);
 	    exit(0);
+	} else if (!strcmp(argv[argc], "-s")) {
+	    pedantic++;			/* strict mode */
 	} else {
 	    usage();
 	}
-    } else if (argc != 1) {
-	usage();
     }
 
-    /* user_pw_ent needs to point to something... */
-    if ((user_pw_ent = getpwuid(getuid())) == NULL) {
-	(void) fprintf(stderr, "%s: Can't find you in the passwd database: ",
+    /* Mock up a fake sudo_user struct. */
+    user_host = user_shost = user_cmnd = "";
+    if ((sudo_user.pw = getpwuid(getuid())) == NULL) {
+	(void) fprintf(stderr, "%s: Can't find you in the passwd database.\n",
 	    Argv[0]);
-	perror(stmp);
 	exit(1);
     }
 
@@ -175,44 +169,55 @@ int main(argc, argv)
 #endif /* ENV_EDITOR */
 
     /*
-     * Copy sudoers file to stmp
+     * Open sudoers temp file and grab a lock.
      */
-    stmp_fd = open(stmp, O_WRONLY | O_CREAT | O_EXCL, 0600);
+    stmp_fd = open(stmp, O_WRONLY | O_CREAT, 0600);
     if (stmp_fd < 0) {
-	if (errno == EEXIST) {
-	    (void) fprintf(stderr, "%s: sudoers file busy, try again later.\n",
-		Argv[0]);
-	    exit(1);
-	}
-	(void) fprintf(stderr, "%s: ", Argv[0]);
-	perror(stmp);
+	(void) fprintf(stderr, "%s: %s\n", Argv[0], strerror(errno));
+	exit(1);
+    }
+    if (!lock_file(stmp_fd, SUDO_TLOCK)) {
+	(void) fprintf(stderr, "%s: sudoers file busy, try again later.\n",
+	    Argv[0]);
+	exit(1);
+    }
+#ifdef HAVE_FTRUNCATE
+    if (ftruncate(stmp_fd, 0) == -1) {
+#else
+    if (truncate(stmp, 0) == -1) {
+#endif
+	(void) fprintf(stderr, "%s: can't truncate %s: %s\n", Argv[0],
+	    stmp, strerror(errno));
 	Exit(-1);
     }
 
-    /* install signal handler to clean up stmp */
+    /* Install signal handlers to clean up stmp if we are killed. */
     setup_signals();
 
+    (void) memset(&sudoers_sb, 0, sizeof(sudoers_sb));
+    if (stat(sudoers, &sudoers_sb) == -1 && errno != ENOENT) {
+	(void) fprintf(stderr, "%s: %s\n", Argv[0], strerror(errno));
+	Exit(-1);
+    }
     sudoers_fd = open(sudoers, O_RDONLY);
-    if (sudoers_fd < 0 && errno != ENOENT) {
-	(void) fprintf(stderr, "%s: ", Argv[0]);
-	perror(sudoers);
+    if (sudoers_fd == -1 && errno != ENOENT) {
+	(void) fprintf(stderr, "%s: %s\n", Argv[0], strerror(errno));
 	Exit(-1);
     }
 
-    /*
-     * Copy the data
-     */
-    if (sudoers_fd >= 0) {
+    /* Copy sudoers -> stmp and reset the mtime */
+    if (sudoers_fd != -1) {
 	while ((n = read(sudoers_fd, buf, sizeof(buf))) > 0)
 	    if (write(stmp_fd, buf, n) != n) {
-		(void) fprintf(stderr, "%s: Write failed: ", Argv[0]);
-		perror("");
+		(void) fprintf(stderr, "%s: Write failed: %s\n", Argv[0],
+		strerror(errno));
 		Exit(-1);
 	    }
 
 	(void) close(sudoers_fd);
     }
     (void) close(stmp_fd);
+    (void) touch(stmp, sudoers_sb.st_mtime);
 
     /*
      * Edit the temp file and parse it (for sanity checking)
@@ -231,21 +236,20 @@ int main(argc, argv)
 	else
 	    (void) sprintf(buf, "%s %s", Editor, stmp);
 
-	/* do the edit -- some SYSV editors exit with 1 instead of 0 */
+	/* Do the edit -- some SYSV editors exit with 1 instead of 0 */
+	now = time(NULL);
 	n = system(buf);
 	if (n != -1 && ((n >> 8) == 0 || (n >> 8) == 1)) {
-	    struct stat statbuf;	/* for sanity checking */
-
-	    /* make sure stmp exists */
-	    if (stat(stmp, &statbuf) < 0) {
+	    /*
+	     * Sanity checks.
+	     */
+	    if (stat(stmp, &stmp_sb) < 0) {
 		(void) fprintf(stderr,
 		    "%s: Can't stat temporary file (%s), %s unchanged.\n",
 		    Argv[0], stmp, sudoers);
 		Exit(-1);
 	    }
-
-	    /* check for zero length file */
-	    if (statbuf.st_size == 0) {
+	    if (stmp_sb.st_size == 0) {
 		(void) fprintf(stderr,
 		    "%s: Zero length temporary file (%s), %s unchanged.\n",
 		    Argv[0], stmp, sudoers);
@@ -253,7 +257,7 @@ int main(argc, argv)
 	    }
 
 	    /*
-	     * passed sanity checks so reopen stmp file and check
+	     * Passed sanity checks so reopen stmp file and check
 	     * for parse errors.
 	     */
 	    yyout = stdout;
@@ -268,10 +272,11 @@ int main(argc, argv)
 		Exit(-1);
 	    }
 
-	    /* clean slate for each parse */
+	    /* Clean slate for each parse */
+	    init_defaults();
 	    init_parser();
 
-	    /* parse the sudoers file */
+	    /* Parse the sudoers file */
 	    if (yyparse() && parse_error != TRUE) {
 		(void) fprintf(stderr,
 		    "%s: Failed to parse temporary file (%s), unknown error.\n",
@@ -286,7 +291,7 @@ int main(argc, argv)
 	}
 
 	/*
-	 * Prompt the user for what to do now
+	 * Got an error, prompt the user for what to do now
 	 */
 	if (parse_error == TRUE) {
 	    switch (whatnow()) {
@@ -295,30 +300,38 @@ int main(argc, argv)
 		case 'x' :	Exit(0);
 				break;
 	    }
+	    yyrestart(yyin);	/* reset lexer */
 	}
     } while (parse_error == TRUE);
 
+    /*
+     * If the user didn't change the temp file, just unlink it.
+     */
+    if (sudoers_sb.st_mtime != now && sudoers_sb.st_mtime == stmp_sb.st_mtime &&
+	sudoers_sb.st_size == stmp_sb.st_size) {
+	(void) fprintf(stderr, "%s: sudoers file unchanged.\n", Argv[0]);
+	Exit(0);
+    }
+
     /*
      * Change mode and ownership of temp file so when
      * we move it to sudoers things are kosher.
      */
     if (chown(stmp, SUDOERS_UID, SUDOERS_GID)) {
 	(void) fprintf(stderr,
-	    "%s: Unable to set (uid, gid) of %s to (%d, %d): ",
-	    Argv[0], stmp, SUDOERS_UID, SUDOERS_GID);
-	perror("");
+	    "%s: Unable to set (uid, gid) of %s to (%d, %d): %s\n",
+	    Argv[0], stmp, SUDOERS_UID, SUDOERS_GID, strerror(errno));
 	Exit(-1);
     }
     if (chmod(stmp, SUDOERS_MODE)) {
 	(void) fprintf(stderr,
-	    "%s: Unable to change mode of %s to %o: ",
-	    Argv[0], stmp, SUDOERS_MODE);
-	perror("");
+	    "%s: Unable to change mode of %s to %o: %s\n",
+	    Argv[0], stmp, SUDOERS_MODE, strerror(errno));
 	Exit(-1);
     }
 
     /*
-     * Now that we have a sane stmp file (parse ok) it needs to be
+     * Now that we have a sane stmp file (parses ok) it needs to be
      * rename(2)'d to sudoers.  If the rename(2) fails we try using
      * mv(1) in case stmp and sudoers are on different filesystems.
      */
@@ -335,9 +348,8 @@ int main(argc, argv)
 		  strlen(sudoers) + 4);
 	    if ((tmpbuf = (char *) malloc(n)) == NULL) {
 		(void) fprintf(stderr,
-			      "%s: Cannot alocate memory, %s unchanged: ",
-			      Argv[0], sudoers);
-		perror("");
+			      "%s: Cannot alocate memory, %s unchanged: %s\n",
+			      Argv[0], sudoers, strerror(errno));
 		Exit(-1);
 	    }
 
@@ -349,11 +361,10 @@ int main(argc, argv)
 			       Argv[0], tmpbuf, sudoers);
 		Exit(-1);
 	    }
-	    (void) free(tmpbuf);
+	    free(tmpbuf);
 	} else {
-	    (void) fprintf(stderr, "%s: Error renaming %s, %s unchanged: ",
-				   Argv[0], stmp, sudoers);
-	    perror("");
+	    (void) fprintf(stderr, "%s: Error renaming %s, %s unchanged: %s\n",
+				   Argv[0], stmp, sudoers, strerror(errno));
 	    Exit(-1);
 	}
     }
@@ -361,68 +372,105 @@ int main(argc, argv)
     return(0);
 }
 
-
-/********************************************************************
- *
- *  dummy *_matches routines
- *
- *  These exist to allow us to use the same parser as sudo(8).
+/*
+ * Dummy *_matches routines.
+ * These exist to allow us to use the same parser as sudo(8).
  */
-
-int command_matches(cmnd, user_args, path, sudoers_args)
+int
+command_matches(cmnd, cmnd_args, path, sudoers_args)
     char *cmnd;
-    char *user_args;
+    char *cmnd_args;
     char *path;
     char *sudoers_args;
 {
     return(TRUE);
 }
 
-
-int addr_matches(n)
+int
+addr_matches(n)
     char *n;
 {
     return(TRUE);
 }
 
-int usergr_matches(g, u)
+int
+usergr_matches(g, u)
     char *g, *u;
 {
     return(TRUE);
 }
 
-
-int netgr_matches(n, h, u)
+int
+netgr_matches(n, h, u)
     char *n, *h, *u;
 {
     return(TRUE);
 }
 
-
-/********************************************************************
- *
- *  usage()
- *
- *  Prints a help message and exits w/ exit value of 1.
+/*
+ * Assuming a parse error occurred, prompt the user for what they want
+ * to do now.  Returns the first letter of their choice.
  */
-
-static void usage()
+static char
+whatnow()
 {
-    (void) fprintf(stderr, "usage: %s [-V]\n", Argv[0]);
-    Exit(-1);
+    int choice, c;
+
+    for (;;) {
+	(void) fputs("What now? ", stdout);
+	choice = getchar();
+	for (c = choice; c != '\n' && c != EOF;)
+	    c = getchar();
+
+	if (choice == 'e' || choice == 'x' || choice == 'Q')
+	    break;
+	else {
+	    (void) puts("Options are:");
+	    (void) puts("  (e)dit sudoers file again");
+	    (void) puts("  e(x)it without saving changes to sudoers file");
+	    (void) puts("  (Q)uit and save changes to sudoers file (DANGER!)\n");
+	}
+    }
+
+    return(choice);
 }
 
-
-/********************************************************************
- *
- *  Exit()
- *
- *  Unlinks the sudoers temp file (if it exists) and exits.
- *  Used in place of a normal exit() and as a signal handler.
- *  A positive parameter is considered to be a signal and is reported.
+/*
+ * Install signal handlers for visudo.
  */
+static void
+setup_signals()
+{
+#ifdef POSIX_SIGNALS
+	struct sigaction action;		/* POSIX signal structure */
+#endif /* POSIX_SIGNALS */
 
-static RETSIGTYPE Exit(sig)
+	/*
+	 * Setup signal handlers to cleanup nicely.
+	 */
+#ifdef POSIX_SIGNALS
+	(void) memset((VOID *)&action, 0, sizeof(action));
+	action.sa_handler = Exit;
+	action.sa_flags = SA_RESETHAND;
+	(void) sigaction(SIGTERM, &action, NULL);
+	(void) sigaction(SIGHUP, &action, NULL);
+	(void) sigaction(SIGINT, &action, NULL);
+	(void) sigaction(SIGQUIT, &action, NULL);
+#else
+	(void) signal(SIGTERM, Exit);
+	(void) signal(SIGHUP, Exit);
+	(void) signal(SIGINT, Exit);
+	(void) signal(SIGQUIT, Exit);
+#endif /* POSIX_SIGNALS */
+}
+
+/*
+ * Unlink the sudoers temp file (if it exists) and exit.
+ * Used in place of a normal exit() and as a signal handler.
+ * A positive parameter is considered to be a signal and is reported.
+ */
+static RETSIGTYPE
+Exit(sig)
     int sig;
 {
     (void) unlink(stmp);
@@ -433,96 +481,9 @@ static RETSIGTYPE Exit(sig)
     exit(-sig);
 }
 
-
-/********************************************************************
- *
- *  whatnow()
- *
- *  Assuming a parse error occurred, prompt the user for what they want
- *  to do now.  Returns first letter of their choice (always lowercase).
- */
-
-static char whatnow()
+static void
+usage()
 {
-    char choice;
-    int ok;
-
-    do {
-	ok = FALSE;
-	(void) fputs("What now? ", stdout);
-	if ((choice = getchar()) != '\n')
-	    while (getchar() != '\n')
-		;
-
-	if (choice == 'e' || choice == 'x' || choice == 'Q')
-	    ok = TRUE;
-
-	/* help message if they gavce us garbage */
-	if (!ok)
-	    whatnow_help();
-
-    } while (!ok);
-
-    return(choice);
-}
-
-
-/********************************************************************
- *
- *  whatnow_help()
- *
- *  Print out a help message for whatnow().
- */
-
-static void whatnow_help()
-{
-    (void) printf("Options are:\n");
-    (void) printf("  (e)dit sudoers file again\n");
-    (void) printf("  e(x)it without saving changes to sudoers file\n");
-    (void) printf("  (Q)uit and save changes to sudoers file (DANGER!)\n\n");
-}
-
-
-/********************************************************************
- *
- *  setup_signals()
- *
- *  Install signal handlers for visudo.
- */
-
-static void setup_signals()
-{
-#ifdef POSIX_SIGNALS
-	struct sigaction action;		/* posix signal structure */
-#endif /* POSIX_SIGNALS */
-
-	/*
-	 * Setup signal handlers
-	 */
-#ifdef POSIX_SIGNALS
-	(void) memset((VOID *)&action, 0, sizeof(action));
-	action.sa_handler = Exit;
-	action.sa_flags = SA_RESETHAND;
-	(void) sigaction(SIGILL, &action, NULL);
-	(void) sigaction(SIGTRAP, &action, NULL);
-	(void) sigaction(SIGBUS, &action, NULL);
-	(void) sigaction(SIGSEGV, &action, NULL);
-	(void) sigaction(SIGTERM, &action, NULL);
-
-	action.sa_handler = SIG_IGN;
-	action.sa_flags = 0;
-	(void) sigaction(SIGHUP, &action, NULL);
-	(void) sigaction(SIGINT, &action, NULL);
-	(void) sigaction(SIGQUIT, &action, NULL);
-#else
-	(void) signal(SIGILL, Exit);
-	(void) signal(SIGTRAP, Exit);
-	(void) signal(SIGBUS, Exit);
-	(void) signal(SIGSEGV, Exit);
-	(void) signal(SIGTERM, Exit);
-
-	(void) signal(SIGHUP, SIG_IGN);
-	(void) signal(SIGINT, SIG_IGN);
-	(void) signal(SIGQUIT, SIG_IGN);
-#endif /* POSIX_SIGNALS */
+    (void) fprintf(stderr, "usage: %s [-s] [-V]\n", Argv[0]);
+    exit(1);
 }
